/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/


////////////////////////////////////////////////////////////////////////////
//
// Assembly Methods
//
////////////////////////////////////////////////////////////////////////////

#include <libc.h>
#include <stdio.h>

#include "oo7.h"
#include "globals.h"
#include "GenParams.h"
#include "VarParams.h"
#include "IntBtree.h"
#include "StringBtree.h"
#include "baidlist.h"

extern int	nextComplexAssemblyId;
extern int	nextBaseAssemblyId;
extern int	debugMode;
extern char*	types[NumTypes];

extern BAIdList* private_cp;
extern BAIdList* shared_cp;

////////////////////////////////////////////////////////////////////////////
//
// ComplexAssembly Constructor
//
////////////////////////////////////////////////////////////////////////////

ComplexAssembly::ComplexAssembly(dbint 		  asId,
				 Module*	  module,
    	                         ComplexAssembly* parentAssembly,
		                 dbint 		  levelNo)
{
    int 		partDate;
    int 		i, nextId;
    ComplexAssembly*	subAssembly;   
    BaseAssembly*	baseAssembly;

    if (debugMode) {
        printf("ComplexAssembly::ComplexAssembly(asId = %d, levelNo = %d)\n",
                asId, levelNo);
    }

    // initialize the simple stuff
    id = asId;
    int typeNo = (int) (random() % NumTypes);
    strncpy(type, types[typeNo], TypeSize);
    buildDate = MinAssmDate + (int) (random() % (MaxAssmDate-MinAssmDate+1));
    superAssembly = parentAssembly;
    subAssemblies = new (ModuleSet) Assoc(&ModuleSet);
    assmType = Complex;

    // recursively create subassemblies for this complex assembly
    for (i = 0; i < NumAssmPerAssm; i++) 
    {
        if (levelNo < NumAssmLevels-1) 
	{
	    // create a complex assembly as the subassembly
            nextId = nextComplexAssemblyId++;
	    subAssembly = new (ModuleSet) ComplexAssembly(nextId, module, 
		this, levelNo+1);
	    subAssemblies->add(subAssembly);
        } 
	else 
	{
	    // create a base assembly as the subassembly
	    // and insert its id into a hash index
            nextId = nextBaseAssemblyId++;
	    baseAssembly = new (ModuleSet) BaseAssembly (nextId, module, this);
	    subAssemblies->add(baseAssembly);
	    BaseAssemblyIdx->insert(nextId, baseAssembly); // add to index
        }
    }
}


////////////////////////////////////////////////////////////////////////////
//
// BaseAssembly Constructor
//
////////////////////////////////////////////////////////////////////////////

BaseAssembly::BaseAssembly(dbint     	asId,
			   Module* 	module,
			   ComplexAssembly* parentAssembly)
{
    CompositePart	*cp;
    int 		compId;
    int 		i;
    int 		lowCompId;
    int 		compIdLimit;


    if (debugMode) {
        printf("BaseAssembly::BaseAssembly(asId = %d, levelNo = %d)\n",
                asId, NumAssmLevels);
    }
    // initialize the simple stuff
    id = asId;
    int typeNo = (int) (random() % NumTypes);
    strncpy(type, types[typeNo], TypeSize);
    buildDate = MinAssmDate + (int) (random() % (MaxAssmDate-MinAssmDate+1));

    superAssembly = parentAssembly;
    superAssembly = parentAssembly;
    componentsPriv = new (ModuleSet) Assoc(&ModuleSet);
    componentsShar = new (ModuleSet) Assoc(&ModuleSet);
    assmType = Base;
    module->allBases->add(this); // add the base assembly to the extent 
		// of all base assemblies in this module

    // get access to the design library containing composite parts
    lowCompId = (module->id - 1) * NumCompPerModule + 1;
    compIdLimit = NumCompPerModule;

    // first select the private composite parts for this assembly
    for (i = 0; i < NumCompPerAssm; i++) 
    {
	compId = lowCompId + (int) (random() % compIdLimit);
	private_cp[compId].insert(this);  // keep track of which
		// composite parts this base assembly uses as private
    }

    // next select the shared composite parts for this assembly
    if (debugMode) { printf("}, Shared parts = { "); }
    for (i = 0; i < NumCompPerAssm; i++) 
    {
	compId = (int) (random() % TotalCompParts) + 1;
	shared_cp[compId].insert(this);  // keep track of which
		// composite parts this base assembly uses as shared
    }
}

//////////////////////////////////////////////////////////////////////////
// 
// DoNothing method for Assemblies 
// 
//////////////////////////////////////////////////////////////////////////

void Assembly::DoNothing ()
{
    if (id < 0) 
        printf("DoNothing: negative id.\n");
    if (debugMode) {
        printf("==> DoNothing(id = %d, type = %s, buildDate = %d)\n",
                               id, type, buildDate);
    }
}
