/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/

#include <libc.h>
#include <stdio.h>

#include "oo7.h"
#include "globals.h"
#include "GenParams.h"
#include "BenchParams.h"
#include <E/sm_btree.h>

//////////////////////////////////////////////////////////////////
//
// Query #2 - 1% selection on AtomicParts via build date (the
// most recent 1% of AtomicParts.)  A B+ tree index is used
//
/////////////////////////////////////////////////////////////////


extern int	debugMode;
extern persistent btree<int> BuildDateIdx;


//////////////////////////////////////////////////////////////////
//
// Routine to do Query #2
//
//////////////////////////////////////////////////////////////////


int query2()
{
    AssocItr* partItr; // iterator for traversing AllParts collection
    AtomicPart* ap;

    // choose a range of dates with the appropriate selectivity
    int dateRange = (Query2Percent * (MaxAtomicDate - MinAtomicDate)) / 100;
    int lowerDate = MaxAtomicDate - dateRange;

    if (debugMode) {
        printf("    Running Query2 with dateRange = %d, lowerDate = %d\n",
		    dateRange, lowerDate);
    }

    int count = 0;
    btree_retrieve<int> DateRetrieve(BuildDateIdx);
    DateRetrieve.SetLB(lowerDate);
    // upper range is end of index?
    for (DateRetrieve.next(); !DateRetrieve.eof; DateRetrieve.next())
    {  
	ap = (AtomicPart *)DateRetrieve.cur_elem; 
	if (debugMode) {
	    printf("    In Query2, partId = %d, buildDate = %d\n",
			ap->id, ap->buildDate);
	}

	// process qualifying parts by calling the null procedure
	if (ap->buildDate >= lowerDate) {
	    ap->DoNothing();
	    count++;
	}
	else // shouldn't happen
	{  
	    printf("error:    In Query2, partId = %d, buildDate = %d, indexval %d lowerDate %d\n",
			ap->id, ap->buildDate, 
			DateRetrieve.cur_keyval,lowerDate);
	}
    }
    return count;
}

