/*
 * stats_server.c
 *
 * Member functions for the ServerStats class.
 */


#include "stats_server.h"


extern "C" int sm_GetRusage ( FLAGS flags, VOLID volid, int *numberOfServers,
			      struct rusage **result );

static esmVolid;

/*
 * The Start() function starts the collecting of stats
 * for a Unix process.
 */
void ServerStats::Start() {
	int numServers;
	struct rusage *rUsage;		/* getrusage() buffer */


	if ( sm_ServerStatistics(VOL_ALL, esmVolid, &numServers,
		&serverStats1, FALSE ) != 0) {
		fprintf(stderr, "ServerStats::Start: sm error\n");
	}
	if ( numServers != 1 ) {
		printf("error: statistics for too many servers(%d).\n", numServers);
		exit(1);
	}


	smPagesRead1 = serverStats1->diskReads;
	smPagesWrite1 = serverStats1->diskWrites;


	if ( sm_GetRusage(VOL_ALL, esmVolid, &numServers, &rUsage ) != 0 ) {
		fprintf(stderr, "ServerStats::Start: sm error\n");
	}
	if ( numServers != 1 ) {
		printf("error: statistics for too many servers(%d).\n", numServers);
		exit(1);
	}
	rusage1 = *rUsage;


	gettimeofday(&time1, NULL);


	free( rUsage );
}


/*
 * The Stop() function stops the collecting of stats
 * for a Unix process.
 */
void ServerStats::Stop() {
	int numServers;
	struct rusage *rUsage;		/* getrusage() buffer */


	/*
	 * Save the current stats in buffer area 2.
	 */
	gettimeofday(&time2, NULL);


	if (sm_GetRusage(VOL_ALL, esmVolid, &numServers, &rUsage) != 0) {
		fprintf(stderr, "ServerStats::Start: sm error\n");
	}
	if ( numServers != 1 ) {
		printf("error: statistics for too many servers(%d).\n", numServers);
		exit(1);
	}
	rusage2 = *rUsage;


	if (sm_ServerStatistics(VOL_ALL, esmVolid, &numServers,
		&serverStats2, FALSE) != 0) {
		fprintf(stderr, "ServerStats::Start: sm error\n");
	}
	if ( numServers != 1 ) {
		printf("error: statistics for too many servers(%d).\n", numServers);
		exit(1);
	}


	smPagesRead2 = serverStats2->diskReads;
	smPagesWrite2 = serverStats2->diskWrites;


	free( serverStats1 );
	free( serverStats2 );
	free( rUsage );


	return;
}


